"""HackerNews MCP Server.

A Model Context Protocol server that provides programmatic access to HackerNews
content through the official HN Algolia API.
"""

__version__ = "1.0.0"

__all__ = ["__version__"]
